@@ -60,6 +60,26 @@ def prepare_directory(destination_directory: str, source_directory: str,
                   os.path.join(destination_directory, 'entrypoint.sh'))
 
 
+def is_docker_installed() -> bool:
+  """Checks if Docker is installed and accessible."""
+  try:
+    docker_client = docker.from_env()
+    logging.info('Local docker: %s', docker_client.version())
+    return True
+  except docker.errors.DockerException as e:
+    if 'No such file or directory' in str(e):
+      # This is the expected case when Docker is not installed, so we don't log
+      # anything, and just return False. The other error branches indicate
+      # something wrong with the Docker installation, so we log an error and
+      # also return False.
+      return False
+    logging.info(e)
+    if 'Permission denied' in str(e):
+      print('Looks like there is a permission problem with docker. '
+            'Did you install sudoless docker?')
+  return False
+
+
 def build_docker_image(image: str,
                        directory: str,
                        dockerfile: Optional[str] = None,
@@ -88,8 +108,8 @@ def push_docker_image(image: str) -> str:
   if not isinstance(push, str) or '"Digest":' not in push:
     raise RuntimeError(
         'Expected docker push to return a string with `status: Pushed` and a '
-        'Digest. This is probably a temporary issue with --build_locally and '
-        'you should try again')
+        'Digest. This is probably a temporary issue with '
+        '--xm_build_image_locally and you should try again')
   # If we are pushing an image, then :latest should also be present.
   docker_client.images.push(repository=repository, tag='latest')
   print('Your image URI is:', termcolor.colored(image, color='blue'))
@@ -121,7 +141,10 @@ def _build_image_with_docker_command(client: docker.DockerClient,
   if progress:
     command[2:2] = ['--progress', 'plain', '--no-cache']
 
-  subprocess.run(command, check=True, env={'DOCKER_BUILDKIT': '1'})
+  subprocess.run(
+      command, check=True, env={
+          **os.environ, 'DOCKER_BUILDKIT': '1'
+      })
 
 
 def _build_image_with_python_client(client: docker.DockerClient, path: str,
