@@ -25,15 +25,14 @@ from xmanager import xm_foo
 import abc
 import asyncio
 from concurrent import futures
-import functools
+import enum
 import getpass
 import inspect
 import queue
 import threading
-from typing import Any, Awaitable, Callable, Collection, Dict, Mapping, Optional, Sequence, overload
+from typing import Any, Awaitable, Callable, Collection, Coroutine, Dict, Generator, List, Mapping, Optional, Sequence, overload
 
 import attr
-import immutabledict
 from xmanager.xm import async_packager
 from xmanager.xm import id_predictor
 from xmanager.xm import job_blocks
@@ -147,7 +146,7 @@ class NotFoundError(KeyError):
 
 def _work_unit_arguments(
     job: job_blocks.JobType,
-    args: Mapping[str, Any],
+    args: Optional[Mapping[str, Any]],
 ) -> Mapping[str, Any]:
   """Constructs work unit arguments to display them in various UIs.
 
@@ -169,7 +168,7 @@ def _work_unit_arguments(
         `Job`s;
       - if it's a job generator, we return `{}`.
   """
-  if args:
+  if args is not None:
     # In order to give users control on what is shown as work unit arguments we
     # don't alter them if a value is given.
     return args
@@ -197,17 +196,48 @@ def _work_unit_arguments(
   return deduce_args(job)
 
 
+class Importance(enum.Enum):
+  """How important it is to schedule particular Experiment or ExperimentUnit.
+
+  This is a hint to the scheduler. Not all schedulers take it into account
+  (xm_local doesn't). And even with smart scheduler a less important work unit
+  may run before a more important one e.g. if it uses a less contended resource.
+
+  Unlike ServiceTier, importance only controls preference within a team i.e. how
+  team's resources are divided between team's experiments. It has no effect on
+  resource allocation between teams.
+  """
+
+  # High impact experiments. Try scheduling them even at the cost of significant
+  # reduction of the overall throughput that your experiments get.
+  HIGH = 'high'
+  # The default importance.
+  NORMAL = 'normal'
+  # Prefer to schedule other experiments with higher importance, but in overall
+  # try to maximize throughput.
+  LOW = 'low'
+
+
+@attr.s(auto_attribs=True, kw_only=True)
 class ExperimentUnitRole(abc.ABC):
-  """The role of an experiment unit within the experiment structure."""
-  pass
+  """The role of an experiment unit within the experiment structure.
+
+  Attributes:
+    importance: how important it is to schedule this executable unit comparing
+      to all your executable units (from all your experiments).
+  """
+  importance: Importance = Importance.NORMAL
 
 
 class ExperimentUnit(abc.ABC):
   """ExperimentUnit is a collection of semantically associated `Job`s."""
 
+  experiment: 'Experiment'
+
   def __init__(self, experiment: 'Experiment',
                create_task: Callable[[Awaitable[Any]], futures.Future],
-               args: Mapping[str, Any], role: ExperimentUnitRole) -> None:
+               args: Optional[Mapping[str,
+                                      Any]], role: ExperimentUnitRole) -> None:
     """Initializes an `ExperimentUnit` instance.
 
     Args:
@@ -217,33 +247,21 @@ class ExperimentUnit(abc.ABC):
         the hyperparameter sweep trial corresponding to a work unit.
       role: The role of this unit in the experiment structure.
     """
-    self._experiment = experiment
+    self.experiment = experiment
     self._create_task = create_task
     self._args = args
     self._role = role
 
-    self._launched_error = None
+    self._launch_tasks: List[futures.Future] = []
 
   @property
   def experiment_id(self) -> int:
     """Returns a unique ID assigned to the experiment."""
-    return self._experiment.experiment_id
+    return self.experiment.experiment_id
 
-  @property
-  @functools.lru_cache()
-  def _launched_event(self) -> asyncio.Event:
-    """Event to be set when the experiment unit is launched."""
-    # In Python < 3.8 `Event` must be created within the event loop. We defer
-    # the initialization because the constructor is not run in the loop. Note
-    # that after migration to Python >= 3.8 this code can be moved to the
-    # constructor.
-    return asyncio.Event()
-
-  def add(
-      self,
-      job: job_blocks.JobType,
-      args: Mapping[str, Any] = immutabledict.immutabledict()
-  ) -> Awaitable[None]:
+  def add(self,
+          job: job_blocks.JobType,
+          args: Optional[Mapping[str, Any]] = None) -> Awaitable[None]:
     # pyformat: disable
     """Adds a Job / JobGroup to the experiment unit.
 
@@ -269,7 +287,8 @@ class ExperimentUnit(abc.ABC):
     """
     # pyformat: enable
     job = job_operators.shallow_copy_job_type(job)
-    _apply_args(job, args)
+    if args is not None:
+      _apply_args(job, args)
     job_operators.populate_job_names(job)
 
     def launch_job(job: job_blocks.Job) -> Awaitable[None]:
@@ -283,43 +302,38 @@ class ExperimentUnit(abc.ABC):
 
     def launch_job_generator(
         job_generator: job_blocks.JobGeneratorType) -> Awaitable[None]:
-      if not inspect.iscoroutinefunction(job_generator):
+      if (not inspect.iscoroutinefunction(job_generator) and
+          not inspect.iscoroutinefunction(job_generator.__call__)):
         raise ValueError(
             'Job generator must be an async function. Signature needs to be '
             '`async def job_generator(work_unit: xm.WorkUnit):`')
-      return job_generator(self, **args)
+      return job_generator(self, **(args or {}))
 
     job_awaitable = pattern_matching.match(launch_job, launch_job_group,
                                            launch_job_generator)(
                                                job)
+    launch_task = self._create_task(job_awaitable)
+    self._launch_tasks.append(launch_task)
+    return asyncio.wrap_future(launch_task)
 
-    async def launch():
-      try:
-        await job_awaitable
-      except Exception as e:
-        self._launched_error = ExperimentUnitError(e)
-        raise
-      finally:
-        # Note that the current implementation reuses `_launched_event`: imagine
-        # a `work_unit.add(generator)` call -- the generator makes a
-        # `work_unit.add` call internally as well, touching `_launched_event`
-        # twice in total. Same would apply for multiple external `work_unit.add`
-        # calls, which is not allowed yet. `_launched_error` is susceptible to
-        # that too.
-        self._launched_event.set()
+  async def _wait_until_complete_impl(self) -> 'ExperimentUnit':
+    try:
+      for task in self._launch_tasks:
+        await asyncio.wrap_future(task)
+    except Exception as e:
+      raise ExperimentUnitError('Experiment unit could not be created.') from e
+    await self._wait_until_complete()
+    return self
 
-    return asyncio.wrap_future(self._create_task(launch()))
-
-  async def wait_until_complete(self):
+  def wait_until_complete(self) -> Awaitable['ExperimentUnit']:
     """Waits until the unit is in a final state: completed/failed/stopped.
 
     Raises:
       ExperimentUnitError: Exception if the unit couldn't complete.
+    Returns:
+      Returns self to facilitate asyncio.as_completed usage.
     """
-    await self._launched_event.wait()
-    if self._launched_error:
-      raise self._launched_error
-    await self._wait_until_complete()
+    return self._wait_until_complete_impl()
 
   async def _launch_job_group(self, job_group: job_blocks.JobGroup,
                               args_view: Mapping[str, Any]) -> None:
@@ -380,7 +394,7 @@ class ExperimentUnit(abc.ABC):
         annotations=metadata_context.ContextAnnotations())
 
 
-@attr.s(auto_attribs=True)
+@attr.s(auto_attribs=True, kw_only=True)
 class WorkUnitRole(ExperimentUnitRole):
   """An experiment unit with this role is a work unit.
 
@@ -390,6 +404,36 @@ class WorkUnitRole(ExperimentUnitRole):
   """
 
 
+class WorkUnitCompletedAwaitable:
+  """Awaitable for work unit completion event.
+
+  Usage:
+    completion_events = [work_unit.wait_until_complete() for work_unit in ...]
+    while completion_events:
+      completed_event, completion_events = asyncio.wait(
+          completion_events, return_when=asyncio.FIRST_COMPLETED)
+      for event in completed_events:
+        wid = event.work_unit.work_unit_id
+        try:
+          await event
+          print(f'Wor unit {wid} completed successfully.')
+        except xm.ExperimentUnitError as e:
+          print(f'Wor unit {wid} failed: {e}.')
+  """
+
+  def __init__(self, work_unit: 'WorkUnit',
+               awaitable: Coroutine[Any, Any, ExperimentUnit]):
+    self.work_unit = work_unit
+    self._awaitable = awaitable
+
+  async def _wait(self) -> 'WorkUnit':
+    await self._awaitable
+    return self.work_unit
+
+  def __await__(self) -> Generator[Any, None, 'WorkUnit']:
+    return self._wait().__await__()
+
+
 class WorkUnit(ExperimentUnit):
   """Work units are experiment units with the work unit role."""
 
@@ -398,8 +442,18 @@ class WorkUnit(ExperimentUnit):
   def work_unit_id(self) -> int:
     raise NotImplementedError
 
+  def wait_until_complete(self) -> WorkUnitCompletedAwaitable:
+    """Waits until the unit is in a final state: completed/failed/stopped.
 
-@attr.s(auto_attribs=True)
+    Raises:
+      ExperimentUnitError: Exception if the unit couldn't complete.
+    Returns:
+      Returns self to facilitate asyncio.as_completed usage.
+    """
+    return WorkUnitCompletedAwaitable(self, self._wait_until_complete_impl())
+
+
+@attr.s(auto_attribs=True, kw_only=True)
 class AuxiliaryUnitRole(ExperimentUnitRole):
   """An experiment unit with this role is an auxiliary unit.
 
@@ -408,13 +462,47 @@ class AuxiliaryUnitRole(ExperimentUnitRole):
   determine the status of the experiment. e.g. Tensorboard
 
   Attributes:
-      termination_delay_secs: How long to keep AUX unit running after experiment
-        completion.
+    termination_delay_secs: How long to keep AUX unit running after experiment
+      completion.
   """
 
   termination_delay_secs: int
 
 
+class AuxiliaryUnitJob(abc.ABC):
+  """A job bundled with an AuxiliaryUnitRole.
+
+  This class allows libraries to define self-contained objects which would
+  result in AUX units once added to the expetiment.
+  Note that this class conforms to xm.JobGenerator interface.
+  """
+
+  role: AuxiliaryUnitRole
+  _job: job_blocks.JobType
+
+  def __init__(self,
+               job: job_blocks.JobType,
+               *,
+               importance: Importance = Importance.NORMAL,
+               termination_delay_secs: int) -> None:
+    self.role = AuxiliaryUnitRole(
+        importance=importance,
+        termination_delay_secs=termination_delay_secs,
+    )
+    self._job = job
+
+  async def __call__(self, aux_unit: ExperimentUnit, **kwargs):
+
+    async def launch_generator(
+        job_generator: job_blocks.JobGeneratorType) -> None:
+      await job_generator(aux_unit, **kwargs)
+
+    async def launch_job(job: Any) -> None:
+      aux_unit.add(job, args=kwargs)
+
+    await pattern_matching.async_match(launch_generator, launch_job)(self._job)
+
+
 class Experiment(abc.ABC):
   """Experiment contains a family of jobs run on the same snapshot of code.
 
@@ -437,12 +525,6 @@ class Experiment(abc.ABC):
     """Returns a unique ID assigned to the experiment."""
     raise NotImplementedError
 
-  def _enter(self) -> None:
-    """Initializes internal state on context manager enter."""
-    self._running_tasks = queue.Queue()
-    self._work_unit_id_predictor = id_predictor.Predictor(1 +
-                                                          self.work_unit_count)
-
   def __enter__(self):
     if asyncio.get_event_loop().is_running():
       raise RuntimeError('When using Experiment from a coroutine please use '
@@ -454,7 +536,14 @@ class Experiment(abc.ABC):
         target=self._event_loop.run_forever, daemon=True)
     self._event_loop_thread.start()
 
-    self._enter()
+    # asyncio.run_coroutine_threadsafe doesn't accept class method and wants it
+    # wrapped in a function.
+    async def async_enter():
+      await self.__aenter__()
+
+    asyncio.run_coroutine_threadsafe(
+        async_enter(), loop=self._event_loop).result()
+
     return self
 
   def _wait_for_tasks(self):
@@ -468,7 +557,9 @@ class Experiment(abc.ABC):
 
   async def __aenter__(self):
     self._event_loop = asyncio.get_event_loop()
-    self._enter()
+    self._running_tasks = queue.Queue()
+    self._work_unit_id_predictor = id_predictor.Predictor(1 +
+                                                          self.work_unit_count)
     return self
 
   async def _await_for_tasks(self):
@@ -482,18 +573,19 @@ class Experiment(abc.ABC):
   def package(
       cls, packageables: Sequence[job_blocks.Packageable] = ()
   ) -> Sequence[job_blocks.Executable]:
-    """Packages executable specs into executables based on the executor specs.
+    """Packages `packageables` & triggers async packaging.
 
-    Builds all given executables specs in parallel. While calling package()
-    multiple times is allowed, that would result in slow sequential build,
-    even if invoked from concurrent threads.
+    This function has 2 usages:
+    - Builds all given executables specs in parallel. While calling package(...)
+      multiple times is allowed, that would result in slow sequential build,
+      even if invoked from concurrent threads.
+    - Triggers packaging of the items enqueued previously with `package_async`.
 
     Args:
-      packageables: A sequence of packageables to build.
+      packageables: A sequence of extra packageables to build synchronously.
 
     Returns:
-      A sequence of packaging results. Order corresponds to the order of input
-      packageables.
+      A sequence of packaging results associated to `packageables` (same order).
     """
     return cls._async_packager.package(packageables)
 
@@ -532,28 +624,42 @@ class Experiment(abc.ABC):
 
   @overload
   def add(self,
-          job: job_blocks.JobType,
-          args: Mapping[str, Any] = ...,
-          role: WorkUnitRole = ...) -> Awaitable[WorkUnit]:
-    ...
-
-  @overload
-  def add(self, job: job_blocks.JobType, args: Mapping[str, Any],
-          role: ExperimentUnitRole) -> Awaitable[ExperimentUnit]:
+          job: AuxiliaryUnitJob,
+          args: Optional[Mapping[str, Any]] = ...) -> Awaitable[ExperimentUnit]:
     ...
 
   @overload
   def add(
       self,
       job: job_blocks.JobType,
-      args: Mapping[str, Any] = ...,
+      args: Optional[Mapping[str, Any]] = ...,
+      *,  # parameters after “*” are keyword-only parameters
+      role: WorkUnitRole = ...
+  ) -> Awaitable[WorkUnit]:
+    ...
+
+  @overload
+  def add(
+      self,
+      job: job_blocks.JobType,
+      args: Optional[Mapping[str, Any]],
+      *,  # parameters after “*” are keyword-only parameters
+      role: ExperimentUnitRole
+  ) -> Awaitable[ExperimentUnit]:
+    ...
+
+  @overload
+  def add(
+      self,
+      job: job_blocks.JobType,
+      args: Optional[Mapping[str, Any]] = ...,
       *,  # parameters after “*” are keyword-only parameters
       role: ExperimentUnitRole
   ) -> Awaitable[ExperimentUnit]:
     ...
 
   # The ExecutableUnit return type is determined by the role.
-  def add(self, job, args=immutabledict.immutabledict(), role=WorkUnitRole()):
+  def add(self, job, args=None, *, role=WorkUnitRole()):
     # pyformat: disable
     """Adds a Job / JobGroup to the experiment.
 
@@ -578,17 +684,38 @@ class Experiment(abc.ABC):
       An awaitable that would be fulfilled when the job is launched.
     """
     # pyformat: enable
-    experiment_unit = self._create_experiment_unit(args, role)
+    role = pattern_matching.match(
+        pattern_matching.Case([AuxiliaryUnitJob], lambda job: job.role),
+        pattern_matching.Case([Any], lambda job: role),
+    )(
+        job)
+    experiment_unit_future = self._create_experiment_unit(args, role)
 
     async def launch():
+      experiment_unit = await experiment_unit_future
       await experiment_unit.add(job, args)
       return experiment_unit
 
     return asyncio.wrap_future(self._create_task(launch()))
 
   @abc.abstractmethod
-  def _create_experiment_unit(self, args: Mapping[str, Any],
-                              role: ExperimentUnitRole) -> ExperimentUnit:
+  def _create_experiment_unit(
+      self, args: Optional[Mapping[str, Any]],
+      role: ExperimentUnitRole) -> Awaitable[ExperimentUnit]:
+    """Creates a new experiment unit.
+
+    Synchronously starts the experiment unit creation, ensuring that IDs would
+    be assigned in invocation order. The operation itself may run asynchronously
+    in background.
+
+    Args:
+      args: Executable unit arguments, to be show as a part of hyper-parameter
+        sweep.
+      role: Executable unit role: whether to create a work or auxiliary unit.
+
+    Returns:
+      An awaitable to the creation result.
+    """
     raise NotImplementedError
 
   def _create_task(self, task: Awaitable[Any]) -> futures.Future:
