@@ -83,7 +83,7 @@ class ServiceTier(enum.Enum, metaclass=_CaseInsensetiveEnumMeta):
   Usually there are at least "Take my money, this workload is important" and
   "Buy excess compute for cheap" offerings. This enum provides a classification
   of such offerings and allows matching comparable (but not necessary identical)
-  options from different runetimes.
+  options from different runtimes.
   """
 
   # Highly available tier. The job is expected to be scheduled fast once sent to
@@ -257,12 +257,12 @@ class Topology:
     self._name = name
 
     dimensions_str = name.split('_')[0]
-    self._dimensions = list(map(int, dimensions_str.split('x')))
+    self.dimensions = list(map(int, dimensions_str.split('x')))
 
   @property
   def chip_count(self) -> int:
     """Returns the number of chips of the TPU topology."""
-    return functools.reduce(operator.mul, self._dimensions)
+    return functools.reduce(operator.mul, self.dimensions)
 
   @property
   def name(self) -> str:
@@ -323,7 +323,7 @@ class JobRequirements:
   topology: Optional[Topology]
 
   location: Optional[str]
-  service_tier: Optional[ServiceTier]
+  service_tier: ServiceTier
 
   def __init__(
       self,
@@ -343,8 +343,9 @@ class JobRequirements:
         Borg cell.
       replicas: Number of identical tasks to run winthin a job. 1 by default.
       service_tier: A service tier at which the job should run.
-      **kw_resources: resource amounts as a kwargs,
-        for example v100=2 or ram=1 * xm.GiB.
+      **kw_resources: resource amounts as a kwargs, for example `v100=2` or
+        `ram=1 * xm.GiB`. See xm.ResourceType enum for the list of supported
+        types and aliases.
 
     Raises:
       ValueError:
@@ -354,9 +355,7 @@ class JobRequirements:
         If topology is supplied for a non acceelerator resource.
     """
     self.location = location
-    self.service_tier = service_tier
-    if self.service_tier is None:
-      self.service_tier = ServiceTier.PROD
+    self.service_tier = service_tier or ServiceTier.PROD
 
     self.task_requirements = ResourceDict()
     self.accelerator = None
@@ -383,9 +382,21 @@ class JobRequirements:
         raise ValueError(f'{resource} has been specified twice.')
       self.task_requirements[resource] = scalar
 
-    if replicas is not None and self.accelerator in TpuType:
-      raise ValueError('Replicated jobs are not supported for TPUs.')
+    if (self.accelerator in GpuType and self.topology and
+        len(self.topology.dimensions) == 2):
+      if replicas is not None:
+        raise ValueError(
+            'Replicated jobs are not supported for multihost GPUs.')
+      replicas = self.topology.dimensions[1]
+
     self.replicas = replicas or 1
+    self._validate_replicas()
+
+  def _validate_replicas(self) -> None:
+    """Raises ValueError if replication is not supported."""
+    if self.replicas > 1 and self.accelerator in TpuType:
+      raise ValueError(
+          f'Replicated jobs are not supported for {self.accelerator}.')
 
   def __repr__(self) -> str:
     """Returns string representation of the requirements."""
