@@ -92,6 +92,7 @@ class JobRequirementsTest(parameterized.TestCase):
     task_requirements = requirements.task_requirements
     self.assertEqual(task_requirements[resources.ResourceType.CPU], 4)
     self.assertEqual(task_requirements[resources.ResourceType.V100], 1)
+    self.assertEqual(requirements.replicas, 1)
 
   def test_resource_specified_twice(self):
     with self.assertRaises(ValueError):
@@ -102,6 +103,22 @@ class JobRequirementsTest(parameterized.TestCase):
     self.assertEqual(requirements.accelerator, resources.ResourceType.TPU_V3)
     self.assertEqual(requirements.topology.name, '4x4')
 
+  def test_multihost_gpu(self):
+    requirements = resources.JobRequirements(v100='4x2')
+    self.assertEqual(requirements.accelerator, resources.ResourceType.V100)
+    self.assertEqual(requirements.topology.name, '4x2')
+    self.assertEqual(requirements.replicas, 2)
+
+  def test_multihost_gpu_with_cpu(self):
+    requirements = resources.JobRequirements(v100='4x2', cpu=1)
+    self.assertEqual(requirements.accelerator, resources.ResourceType.V100)
+    self.assertEqual(requirements.topology.name, '4x2')
+    self.assertEqual(requirements.replicas, 2)
+
+  def test_multihost_gpu_with_replicas(self):
+    with self.assertRaises(ValueError):
+      resources.JobRequirements(v100='4x2', replicas=3)
+
   def test_location(self):
     requirements = resources.JobRequirements(location='lon_r7')
     self.assertEqual(requirements.location, 'lon_r7')
@@ -116,7 +133,7 @@ class JobRequirementsTest(parameterized.TestCase):
     self.assertEqual(requirements.replicas, 2)
 
     with self.assertRaises(ValueError):
-      resources.JobRequirements(replicas=2, tpu_v3='1x1')
+      resources.JobRequirements(replicas=2, tpu_v3='4x4')
 
   def test_str(self):
     self.assertEqual(
