@@ -13,16 +13,18 @@
 # limitations under the License.
 """Utilities for testing core objects."""
 
+import asyncio
 from concurrent import futures
-from typing import Any, Awaitable, Callable, List, Mapping
+from typing import Any, Awaitable, Callable, List, Mapping, Optional
 
 import attr
 from xmanager.xm import core
 from xmanager.xm import id_predictor
 from xmanager.xm import job_blocks
+from xmanager.xm import pattern_matching as pm
 
 
-class TestWorkUnit(core.WorkUnit):
+class TestExperimentUnit(core.WorkUnit):
   """A test version of WorkUnit with abstract methods implemented."""
 
   def __init__(
@@ -31,10 +33,11 @@ class TestWorkUnit(core.WorkUnit):
       work_unit_id_predictor: id_predictor.Predictor,
       create_task: Callable[[Awaitable[Any]], futures.Future],
       launched_jobs: List[job_blocks.JobType],
-      launched_jobs_args: List[Mapping[str, Any]],
-      args: Mapping[str, Any],
+      launched_jobs_args: List[Optional[Mapping[str, Any]]],
+      args: Optional[Mapping[str, Any]],
+      role: core.ExperimentUnitRole,
   ) -> None:
-    super().__init__(experiment, create_task, args, core.WorkUnitRole())
+    super().__init__(experiment, create_task, args, role)
     self._launched_jobs = launched_jobs
     self._launched_jobs_args = launched_jobs_args
     self._work_unit_id = work_unit_id_predictor.reserve_id()
@@ -43,7 +46,7 @@ class TestWorkUnit(core.WorkUnit):
     """Test work unit is immediately complete."""
 
   async def _launch_job_group(self, job_group: job_blocks.JobGroup,
-                              args: Mapping[str, Any]) -> None:
+                              args: Optional[Mapping[str, Any]]) -> None:
     """Appends the job group to the launched_jobs list."""
     self._launched_jobs.extend(job_group.jobs.values())
     self._launched_jobs_args.append(args)
@@ -67,15 +70,24 @@ class TestExperiment(core.Experiment):
     self.launched_jobs = []
     self.launched_jobs_args = []
     self._work_units = []
+    self._auxiliary_units = []
 
-  def _create_experiment_unit(self, args,
-                              role=core.WorkUnitRole()) -> TestWorkUnit:
+  def _create_experiment_unit(
+      self, args, role=core.WorkUnitRole()) -> Awaitable[TestExperimentUnit]:
     """Creates a new WorkUnit instance for the experiment."""
-    work_unit = TestWorkUnit(self, self._work_unit_id_predictor,
-                             self._create_task, self.launched_jobs,
-                             self.launched_jobs_args, args)
-    self._work_units.append(work_unit)
-    return work_unit
+    future = asyncio.Future()
+    experiment_unit = TestExperimentUnit(self, self._work_unit_id_predictor,
+                                         self._create_task, self.launched_jobs,
+                                         self.launched_jobs_args, args, role)
+    pm.match(
+        pm.Case([core.WorkUnitRole],
+                lambda _: self._work_units.append(experiment_unit)),
+        pm.Case([core.AuxiliaryUnitRole],
+                lambda _: self._auxiliary_units.append(experiment_unit)))(
+                    role)
+
+    future.set_result(experiment_unit)
+    return future
 
   @property
   def work_unit_count(self) -> int:
@@ -85,6 +97,10 @@ class TestExperiment(core.Experiment):
   def work_units(self):
     return self._work_units
 
+  @property
+  def auxiliary_units(self):
+    return self._auxiliary_units
+
   @property
   def experiment_id(self) -> int:
     return 1
